###
## SecClabCa
## Copyright (C) 2003 SecClab group (mail. lfp70@hotmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public Licence as published by
# the Free Software Foundation; eihter version 2 of the License, or
# (at your option) any later version.
#
# This program is distributes in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
##

from ConfigParser import ConfigParser
from ConfigParser import DuplicateSectionError

SECTION_CA                     = 'ca'
SECTION_CACONF                 = 'ca_conf'
SECTION_POLICYMATCH            = 'policy_match'
SECTION_POLICYANY              = 'policy_any'
SECTION_REQ                    = 'req'
SECTION_NEWOIDS                = 'new_oids'
SECTION_REQDN                  = 'req_distinguished_name'
SECTION_REQATTR                = 'req_attributes'
SECTION_ENGINE                 = 'engine'

OPTION_HOME                  = 'HOME'
OPTION_RANDFILE              = 'RANDFILE'
OPTION_OIDFILE               = 'oid_file'
OPTION_OIDSECTION            = 'oid_section'

OPTION_CACONF_DIR             = 'dir'
OPTION_CACONF_CERTS           = 'certs'
OPTION_CACONF_CRLDIR          = 'crl_dir'
OPTION_CACONF_DATABASE        = 'database'
OPTION_CACONF_NEWCERTSDIR     = 'new_certs_dir'
OPTION_CACONF_CERTIFICATE     = 'certificate'
OPTION_CACONF_SERIAL          = 'serial'
OPTION_CACONF_CRL             = 'crl'
OPTION_CACONF_PRIVATEKEY      = 'private_key'
OPTION_CACONF_RANDFILE        = 'RANDFILE'
OPTION_CACONF_X509EXTENSIONS  = 'x509_extensions'
OPTION_CACONF_DEFAULTDAYS     = 'default_days'
OPTION_CACONF_DEFAULTCRLDAYS  = 'default_crl_days'
OPTION_CACONF_DEFAULTMD       = 'default_md'
OPTION_CACONF_PRESERVE        = 'preserve'
OPTION_CACONF_POLICY          = 'policy'
OPTION_CACONF_SECCLABPOLICIES = 'secclab_policies'
OPTION_CACONF_REQ             = 'req'

OPTION_POLICY_COUNTRYNAME           = 'countryName'
OPTION_POLICY_STATEORPROVINCENAME   = 'stateOrProvinceName'
OPTION_POLICY_LOCALITYNAME          = 'localityName'
OPTION_POLICY_ORGANIZATIONNAME      = 'organizationName'
OPTION_POLICY_ORGANIZATIONUNITNAME  = 'organizationUnitName'
OPTION_POLICY_COMMONNAME            = 'commonName'
OPTION_POLICY_EMAILADDRESS          = 'emailAddress'

VALUE_POLICY_MATCH    = 'match'
VALUE_POLICY_OPTIONAL = 'optional'
VALUE_POLICY_SUPPLIED = 'supplied'

OPTION_REQ_DEFAULTBITS       = 'default_bits'
OPTION_REQ_DEFAULTKEYFILE    = 'default_keyfile'
OPTION_REQ_DISTINGUISHEDNAME = 'distinguished_name'
OPTION_REQ_ATTRIBUTES        = 'attributes'
OPTION_REQ_X509EXTENSIONS    = 'x509_extensions'
OPTION_REQ_PROMPT            = 'prompt'

OPTION_REQATTR_CHALLENGEPASSWORD = 'challengePassword'
OPTION_REQATTR_UNSTRUCTUREDNAME  = 'unstructuredName'

SUFFIX_REQDN_DEFAULT = 'default'
SUFFIX_REQDN_MAX     = 'max'
SUFFIX_REQDN_MIN     = 'min'
SUFFIX_REQDN_REGEX   = 'regex'

OPTION_CA_DEFAULTCA = 'default_ca'

CONFIG_FIRST_LINE           = "### DON'T EDIT THIS FILE. IT IS AUTOMATICALY GENERATED"
CONFIG_INIT_VARIABLE_LIST   = "# INIT VARIABLE LIST"
CONFIG_FINISH_VARIABLE_LIST = "# FINISH VARIABLE LIST"

class OpensslConfFile(ConfigParser):
    class File:
        def __init__(self,filename,mode='r'):
            self.filename = filename
            self.file     = open(filename,mode)
            self.give_next_default_line = 0
            
        def readline(self):
            if self.give_next_default_line == 1:
                self.give_next_default_line = 0
                return "[DEFAULT]"
            return self.file.readline()
    
        def write(self,s):
            if s[0:9] != '[DEFAULT]':
                self.file.write(s)
        def __del__(self):
            self.file.close()
    def __init__(self):
        ConfigParser.__init__(self)
        self.variable_list = {}

    def _read_variable_hack(self,fp):
        line = fp.readline()
        if line != CONFIG_FIRST_LINE:
            raise "CONFIG_ERROR","NOT VALID CONFIG FILE"
        line = fp.readline()
        if line != CONFIG_INIT_VARIABLE_LIST:
            raise "CONFIG_ERROR","NOT VALID CONFIG FILE"
        line = fp.readline()
        while line:
            if line == CONFIG_FINISH_VARIABLE_LIST:
                break
            var,val = line.split(string.strip(line),'=',2)
            var = var.strip()
            val = val.strip()
            self.variable_list[var] = val
        
    def _write_variable_hack(self,fp):
        fp.write(CONFIG_FIRST_LINE)
        fp.write(CONFIG_INIT_VARIABLE_LIST)
        for (var,val) in self.variable_list.items():
            fp.write("%s = %s\n" % (var,val))
        fp.write(CONFIG_FINISH_VARIABLE_LIST)
        
    def add_variable(self,var,val):
        self.variable_list[var] = val
        
    def add_section(self,section):
        if section == 'DEFAULT':
            raise DuplicateSectionError(section)
        ConfigParser.add_section(self,section)

    def readfp(self,filename):
        file_fp = self.File(filename,'r')
        self._read_variable_hack(file_fp)
        file_fp.give_next_default_line = 1
        ConfigParser.readfp(self,file_fp)

    def read(self,files):
        raise "ERROR","ERROR NOT IMPLEMENTED"

    def write(self,filename):
        file_fp = self.File(filename,'w')
        self._write_variable_hack(file_fp)
        ConfigParser.write(file_fp)

    def optionxform(self, optionstr):
        return optionstr
